import { MuSchema } from 'muschema/schema';
import { MuStruct } from 'muschema/struct';
import { MuUint32 } from 'muschema/uint32';
import { MuString } from 'muschema/string';
import { MuRemoteRPCClient } from './server';
export declare type MuAnySchema = MuSchema<any>;
export declare type MuRPCSchema = [MuAnySchema, MuAnySchema];
export interface MuRPCTable {
    [method: string]: MuRPCSchema;
}
export interface MuRPCProtocolSchema {
    client: MuRPCTable;
    server: MuRPCTable;
}
export declare type MuRPCError = string;
export interface MuRPCInterface<RPCTable extends MuRPCTable> {
    callerAPI: {
        [method in keyof RPCTable]: (arg: RPCTable[method][0]['identity'], callback?: (response: RPCTable[method][1]['identity']) => void) => void;
    };
    clientHandlerAPI: {
        [method in keyof RPCTable]: (arg: RPCTable[method][0]['identity'], next: (err: MuRPCError | undefined, response?: RPCTable[method][1]['identity']) => void) => void;
    };
    serverHandlerAPI: {
        [method in keyof RPCTable]: (arg: RPCTable[method][0]['identity'], next: (err: MuRPCError | undefined, response?: RPCTable[method][1]['identity']) => void, client?: MuRemoteRPCClient<RPCTable>) => void;
    };
}
export declare type MuRPCProtocolTablePhase<RPCTable extends MuRPCTable, Phase extends 0 | 1> = {
    [method in keyof RPCTable]: MuStruct<{
        base: RPCTable[method][Phase];
        id: MuUint32;
    }>;
};
export declare type MuRPCProtocolSchemaUnfolded<ProtocolSchema extends MuRPCProtocolSchema> = [{
    client: MuRPCProtocolTablePhase<ProtocolSchema['client'], 0>;
    server: MuRPCProtocolTablePhase<ProtocolSchema['server'], 0>;
}, {
    client: MuRPCProtocolTablePhase<ProtocolSchema['server'], 1>;
    server: MuRPCProtocolTablePhase<ProtocolSchema['client'], 1>;
}];
export declare function unfoldRPCProtocolSchema<ProtocolSchema extends MuRPCProtocolSchema>(schema: ProtocolSchema): MuRPCProtocolSchemaUnfolded<ProtocolSchema>;
export declare function MuRPC<ArgumentSchema extends MuAnySchema, ResponseSchema extends MuAnySchema>(arg: ArgumentSchema, response: ResponseSchema): [ArgumentSchema, ResponseSchema];
export declare const MuRPCErrorSchema: MuStruct<{
    message: MuString;
    id: MuUint32;
}>;
export declare const MuRPCErrorProtocolSchema: {
    client: {
        error: MuStruct<{
            message: MuString;
            id: MuUint32;
        }>;
    };
    server: {
        error: MuStruct<{
            message: MuString;
            id: MuUint32;
        }>;
    };
};
